"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useInOutMode = void 0;
const react_1 = require("react");
const setAnimationFrameTimeout_1 = require("../helpers/setAnimationFrameTimeout");
function useInOutMode({ state, timeout, mode, keyRef, list, setList, }) {
    const timerRef = (0, react_1.useRef)({});
    const timerRef2 = (0, react_1.useRef)({});
    (0, react_1.useEffect)(() => {
        // skip unmatched mode 🚫
        if (mode !== 'in-out')
            return;
        const [lastItem, secondLastItem] = list.reverse();
        // if state has changed && stage is enter (add new item)
        if (lastItem.state !== state && lastItem.stage === 'enter') {
            // 1 add new item with stage 'from'
            keyRef.current++;
            setList((prev) => prev.slice(-1).concat({ state, key: keyRef.current, stage: 'from' }));
        }
        // if state hasn't changed && stage is from (enter that new item)
        if (lastItem.state === state && lastItem.stage === 'from') {
            // 2 set that new item's stage to 'enter' immediately
            (0, setAnimationFrameTimeout_1.setAnimationFrameTimeout)(() => {
                setList([secondLastItem, Object.assign(Object.assign({}, lastItem), { stage: 'enter' })]);
            });
        }
        // if state hasn't changed
        // && stage is enter
        // && second last item exist
        // && second last item enter
        // (leave second last item)
        if (lastItem.state === state &&
            lastItem.stage === 'enter' &&
            secondLastItem &&
            (secondLastItem === null || secondLastItem === void 0 ? void 0 : secondLastItem.stage) === 'enter') {
            // 3 leave second last item after new item enter animation ends
            (0, setAnimationFrameTimeout_1.clearAnimationFrameTimeout)(timerRef.current);
            timerRef.current = (0, setAnimationFrameTimeout_1.setAnimationFrameTimeout)(() => {
                setList([Object.assign(Object.assign({}, secondLastItem), { stage: 'leave' }), lastItem]);
            }, timeout);
        }
        // if second last item exist
        // && second last item is enter
        // (unmount second last item)
        if (secondLastItem && secondLastItem.stage === 'leave') {
            // 4 unmount second last item after it's leave animation ends
            (0, setAnimationFrameTimeout_1.clearAnimationFrameTimeout)(timerRef2.current);
            timerRef2.current = (0, setAnimationFrameTimeout_1.setAnimationFrameTimeout)(() => {
                setList([lastItem]);
            }, timeout);
        }
        return () => {
            (0, setAnimationFrameTimeout_1.clearAnimationFrameTimeout)(timerRef.current);
            (0, setAnimationFrameTimeout_1.clearAnimationFrameTimeout)(timerRef2.current);
        };
    }, [keyRef, list, mode, setList, state, timeout]);
}
exports.useInOutMode = useInOutMode;
//# sourceMappingURL=useInOutMode.js.map